@@ -244,8 +244,6 @@ module Cinch
 
         if msg.command == "PRIVMSG"
           events << [:message]
-        else
-          events << [:notice]
         end
 
         if msg.action?
@@ -364,11 +362,11 @@ module Cinch
     def on_away(msg, events)
       if msg.message.to_s.empty?
         # unaway
-        m.user.sync(:away, nil, true)
+        msg.user.sync(:away, nil, true)
         events << [:unaway]
       else
         # away
-        m.user.sync(:away, msg.message, true)
+        msg.user.sync(:away, msg.message, true)
         events << [:away]
       end
     end
@@ -385,7 +383,7 @@ module Cinch
         else
           send_cap_end
         end
-      when "NACK"
+      when "NAK"
         send_cap_end
       end
     end
@@ -492,11 +490,14 @@ module Cinch
 
     def on_nick(msg, events)
       if msg.user == @bot
-        @bot.set_nick msg.params.last
+        # @bot.set_nick msg.params.last
+        target = @bot
+      else
+        target = msg.user
       end
 
-      msg.user.update_nick(msg.params.last)
-      msg.user.online = true
+      target.update_nick(msg.params.last)
+      target.online = true
     end
 
     def on_part(msg, events)
@@ -543,15 +544,23 @@ module Cinch
       end
 
 
-      if msg.message =~ /^\001DCC SEND (\S+) (\d+) (\d+)(?: (\d+))?\001$/
-        process_dcc_send($1, $2, $3, $4, msg, events)
+      if msg.message =~ /^\001DCC SEND (?:"([^"]+)"|(\S+)) (\S+) (\d+)(?: (\d+))?\001$/
+        process_dcc_send($1 || $2, $3, $4, $5, msg, events)
       end
     end
 
     # @since 2.0.0
     def process_dcc_send(filename, ip, port, size, m, events)
-      ip   = ip.to_i
-      ip   = [24, 16, 8, 0].collect {|b| (ip >> b) & 255}.join('.')
+      if ip =~ /^\d$/
+        # If ip is a single integer, assume it's a specification
+        # compliant IPv4 address in network byte order. If it's any
+        # other string, assume that it's a valid IPv4 or IPv6 address.
+        # If it's not valid, let someone higher up the chain notice
+        # that.
+        ip   = ip.to_i
+        ip   = [24, 16, 8, 0].collect {|b| (ip >> b) & 255}.join('.')
+      end
+
       port = port.to_i
       size = size.to_i
 
@@ -631,8 +640,8 @@ module Cinch
     def on_319(msg, events)
       # RPL_WHOISCHANNELS
       user     = User(msg.params[1])
-      channels = msg.params[2].scan(/#{@isupport["CHANTYPES"].join}[^ ]+/o).map {|c| Channel(c) }
-      user.sync(:channels, channels, true)
+      channels = msg.params[2].scan(/[#{@isupport["CHANTYPES"].join}][^ ]+/o).map {|c| Channel(c) }
+      @whois_updates[user].merge!({:channels => channels})
     end
 
     def on_324(msg, events)
