@@ -1,4 +1,5 @@
 module Cinch
+  # Provide blocking access to user/channel information.
   module Syncable
     # Blocks until the object is synced.
     #
@@ -6,8 +7,20 @@ module Cinch
     # @api private
     def wait_until_synced(attr)
       attr = attr.to_sym
+      waited = 0
       while true
-        return if @synced_attributes.include?(attr)
+        return if synced?(attr)
+        waited += 1
+
+        if waited % 100 == 0
+          bot.loggers.warn "A synced attribute ('%s' for %s) has not been available for %d seconds, still waiting" % [attr, self.inspect, waited / 10]
+          bot.loggers.warn caller.map {|s| "  #{s}"}
+
+          if waited / 10 >= 30
+            bot.loggers.warn "  Giving up..."
+            raise Exceptions::SyncedAttributeNotAvailable, "'%s' for %s" % [attr, self.inspect]
+          end
+        end
         sleep 0.1
       end
     end
@@ -42,6 +55,9 @@ module Cinch
       @synced_attributes.clear
     end
 
+    # @param [Symbol] attribute
+    # @param [Boolean] data
+    # @param [Boolean] unsync
     # @api private
     def attr(attribute, data = false, unsync = false)
       unless unsync
